@@ -52,3 +52,16 @@ with ((((open("bla.txt")))) as f):
 
 with ((((CtxManager1()))) as example1, (((CtxManager2()))) as example2):
     ...
+
+# regression tests for #3678
+with (a, *b):
+    pass
+
+with (a, (b, *c)):
+    pass
+
+with (a for b in c):
+    pass
+
+with (a, (b for c in d)):
+    pass
