@@ -20,7 +20,6 @@ use crate::expression::{
 };
 use crate::other::interpolated_string::InterpolatedStringLayout;
 use crate::prelude::*;
-use crate::preview::is_parenthesize_lambda_bodies_enabled;
 use crate::statement::trailing_semicolon;
 use crate::string::StringLikeExtensions;
 use crate::string::implicit::{
@@ -1355,7 +1354,7 @@ fn is_attribute_with_parenthesized_value(target: &Expr, context: &PyFormatContex
     }
 }
 
-/// Like [`maybe_parenthesize_expression`] but with special handling for lambdas in preview.
+/// Like [`maybe_parenthesize_expression`] but with special handling for lambdas.
 fn maybe_parenthesize_value<'a>(
     expression: &'a Expr,
     parent: AnyNodeRef<'a>,
@@ -1372,8 +1371,7 @@ impl Format<PyFormatContext<'_>> for MaybeParenthesizeValue<'_> {
     fn fmt(&self, f: &mut PyFormatter) -> FormatResult<()> {
         let MaybeParenthesizeValue { expression, parent } = self;
 
-        if is_parenthesize_lambda_bodies_enabled(f.context())
-            && let Expr::Lambda(lambda) = expression
+        if let Expr::Lambda(lambda) = expression
             && !f.context().comments().has_leading(lambda)
         {
             parenthesize_if_expands(&lambda.format().with_options(ExprLambdaLayout::Assignment))
