@@ -5,7 +5,7 @@ use super::{DeferredExpressionState, TypeInferenceBuilder};
 use crate::FxOrderSet;
 use crate::semantic_index::semantic_index;
 use crate::types::diagnostic::{
-    self, INVALID_TYPE_FORM, NON_SUBSCRIPTABLE, report_invalid_argument_number_to_special_form,
+    self, INVALID_TYPE_FORM, NOT_SUBSCRIPTABLE, report_invalid_argument_number_to_special_form,
     report_invalid_arguments_to_callable,
 };
 use crate::types::generics::bind_typevar;
@@ -16,8 +16,8 @@ use crate::types::tuple::{TupleSpecBuilder, TupleType};
 use crate::types::{
     BindingContext, CallableType, DynamicType, GenericContext, IntersectionBuilder, KnownClass,
     KnownInstanceType, LintDiagnosticGuard, Parameter, Parameters, SpecialFormType, SubclassOfType,
-    Type, TypeAliasType, TypeContext, TypeIsType, TypeMapping, TypeVarKind, UnionBuilder,
-    UnionType, any_over_type, todo_type,
+    Type, TypeAliasType, TypeContext, TypeGuardType, TypeIsType, TypeMapping, TypeVarKind,
+    UnionBuilder, UnionType, any_over_type, todo_type,
 };
 
 /// Type expressions
@@ -173,10 +173,6 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 }
             }
 
-            // Avoid inferring the types of invalid type expressions that have been parsed from a
-            // string annotation, as they are not present in the semantic index.
-            _ if self.deferred_state.in_string_annotation() => Type::unknown(),
-
             // =====================================================================================
             // Forms which are invalid in the context of annotation expressions: we infer their
             // nested expressions as normal expressions, but the type of the top-level expression is
@@ -308,7 +304,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::BoolOp(bool_op) => {
-                self.infer_boolean_expression(bool_op);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_boolean_expression(bool_op);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Boolean operations are not allowed in type expressions"),
@@ -317,7 +315,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Named(named) => {
-                self.infer_named_expression(named);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_named_expression(named);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Named expressions are not allowed in type expressions"),
@@ -326,7 +326,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::UnaryOp(unary) => {
-                self.infer_unary_expression(unary);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_unary_expression(unary);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Unary operations are not allowed in type expressions"),
@@ -335,7 +337,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Lambda(lambda_expression) => {
-                self.infer_lambda_expression(lambda_expression);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_lambda_expression(lambda_expression);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("`lambda` expressions are not allowed in type expressions"),
@@ -344,7 +348,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::If(if_expression) => {
-                self.infer_if_expression(if_expression, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_if_expression(if_expression, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("`if` expressions are not allowed in type expressions"),
@@ -353,7 +359,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Dict(dict) => {
-                self.infer_dict_expression(dict, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_dict_expression(dict, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Dict literals are not allowed in type expressions"),
@@ -362,7 +370,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Set(set) => {
-                self.infer_set_expression(set, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_set_expression(set, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Set literals are not allowed in type expressions"),
@@ -371,7 +381,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::DictComp(dictcomp) => {
-                self.infer_dict_comprehension_expression(dictcomp, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_dict_comprehension_expression(dictcomp, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Dict comprehensions are not allowed in type expressions"),
@@ -380,7 +392,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::ListComp(listcomp) => {
-                self.infer_list_comprehension_expression(listcomp, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_list_comprehension_expression(listcomp, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("List comprehensions are not allowed in type expressions"),
@@ -389,7 +403,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::SetComp(setcomp) => {
-                self.infer_set_comprehension_expression(setcomp, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_set_comprehension_expression(setcomp, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Set comprehensions are not allowed in type expressions"),
@@ -398,7 +414,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Generator(generator) => {
-                self.infer_generator_expression(generator);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_generator_expression(generator);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Generator expressions are not allowed in type expressions"),
@@ -407,7 +425,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Await(await_expression) => {
-                self.infer_await_expression(await_expression);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_await_expression(await_expression);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("`await` expressions are not allowed in type expressions"),
@@ -416,7 +436,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Yield(yield_expression) => {
-                self.infer_yield_expression(yield_expression);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_yield_expression(yield_expression);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("`yield` expressions are not allowed in type expressions"),
@@ -425,7 +447,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::YieldFrom(yield_from) => {
-                self.infer_yield_from_expression(yield_from);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_yield_from_expression(yield_from);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("`yield from` expressions are not allowed in type expressions"),
@@ -434,7 +458,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Compare(compare) => {
-                self.infer_compare_expression(compare);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_compare_expression(compare);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Comparison expressions are not allowed in type expressions"),
@@ -443,7 +469,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Call(call_expr) => {
-                self.infer_call_expression(call_expr, TypeContext::default());
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_call_expression(call_expr, TypeContext::default());
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Function calls are not allowed in type expressions"),
@@ -452,7 +480,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::FString(fstring) => {
-                self.infer_fstring_expression(fstring);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_fstring_expression(fstring);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("F-strings are not allowed in type expressions"),
@@ -461,7 +491,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::TString(tstring) => {
-                self.infer_tstring_expression(tstring);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_tstring_expression(tstring);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("T-strings are not allowed in type expressions"),
@@ -470,7 +502,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
 
             ast::Expr::Slice(slice) => {
-                self.infer_slice_expression(slice);
+                if !self.deferred_state.in_string_annotation() {
+                    self.infer_slice_expression(slice);
+                }
                 self.report_invalid_type_expression(
                     expression,
                     format_args!("Slices are not allowed in type expressions"),
@@ -520,7 +554,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
     ) -> Type<'db> {
         match value_ty {
             Type::ClassLiteral(class_literal) => match class_literal.known(self.db()) {
-                Some(KnownClass::Tuple) => Type::tuple(self.infer_tuple_type_expression(slice)),
+                Some(KnownClass::Tuple) => Type::tuple(self.infer_tuple_type_expression(subscript)),
                 Some(KnownClass::Type) => self.infer_subclass_of_type_expression(slice),
                 _ => self.infer_subscript_type_expression(subscript, value_ty),
             },
@@ -549,10 +583,13 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
         }
     }
 
-    /// Given the slice of a `tuple[]` annotation, return the type that the annotation represents
+    /// Return the type represented by a `tuple[]` expression in a type annotation.
+    ///
+    /// This method assumes that a type has already been inferred and stored for the `value`
+    /// of the subscript passed in.
     pub(super) fn infer_tuple_type_expression(
         &mut self,
-        tuple_slice: &ast::Expr,
+        tuple: &ast::ExprSubscript,
     ) -> Option<TupleType<'db>> {
         /// In most cases, if a subelement of the tuple is inferred as `Todo`,
         /// we should only infer `Todo` for that specific subelement.
@@ -575,13 +612,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                     element_ty.exact_tuple_instance_spec(builder.db()).is_none()
                 }
                 ast::Expr::Subscript(ast::ExprSubscript { value, .. }) => {
-                    let value_ty = if builder.deferred_state.in_string_annotation() {
-                        // Using `.expression_type` does not work in string annotations, because
-                        // we do not store types for sub-expressions. Re-infer the type here.
-                        builder.infer_expression(value, TypeContext::default())
-                    } else {
-                        builder.expression_type(value)
-                    };
+                    let value_ty = builder.expression_type(value);
 
                     value_ty == Type::SpecialForm(SpecialFormType::Unpack)
                 }
@@ -589,14 +620,22 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             }
         }
 
-        // TODO: PEP 646
-        match tuple_slice {
+        // TODO: TypeVarTuple
+        match &*tuple.slice {
             ast::Expr::Tuple(elements) => {
                 if let [element, ellipsis @ ast::Expr::EllipsisLiteral(_)] = &*elements.elts {
+                    if element.is_starred_expr()
+                        && let Some(builder) = self.context.report_lint(&INVALID_TYPE_FORM, tuple)
+                    {
+                        let mut diagnostic =
+                            builder.into_diagnostic("Invalid `tuple` specialization");
+                        diagnostic
+                            .set_primary_message("`...` cannot be used after an unpacked element");
+                    }
                     self.infer_expression(ellipsis, TypeContext::default());
                     let result =
                         TupleType::homogeneous(self.db(), self.infer_type_expression(element));
-                    self.store_expression_type(tuple_slice, Type::tuple(Some(result)));
+                    self.store_expression_type(&tuple.slice, Type::tuple(Some(result)));
                     return Some(result);
                 }
 
@@ -606,14 +645,65 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 // (see docstring for `element_could_alter_type_of_whole_tuple`)
                 let mut return_todo = false;
 
+                let mut first_unpacked_variadic_tuple = None;
+
                 for element in elements {
+                    if element.is_ellipsis_literal_expr()
+                        && let Some(builder) = self.context.report_lint(&INVALID_TYPE_FORM, tuple)
+                    {
+                        let mut diagnostic =
+                            builder.into_diagnostic("Invalid `tuple` specialization");
+                        diagnostic.set_primary_message(
+                            "`...` can only be used as the second element \
+                            in a two-element `tuple` specialization",
+                        );
+                    }
                     let element_ty = self.infer_type_expression(element);
                     return_todo |=
                         element_could_alter_type_of_whole_tuple(element, element_ty, self);
 
-                    if let ast::Expr::Starred(_) = element {
+                    if let ast::Expr::Starred(ast::ExprStarred {
+                        value: starred_value,
+                        ..
+                    }) = element
+                    {
+                        let mut report_too_many_unpacked_tuples = || {
+                            if let Some(first_unpacked_variadic_tuple) =
+                                first_unpacked_variadic_tuple
+                            {
+                                if let Some(builder) =
+                                    self.context.report_lint(&INVALID_TYPE_FORM, tuple)
+                                {
+                                    let mut diagnostic = builder.into_diagnostic(
+                                        "Multiple unpacked variadic tuples \
+                                            are not allowed in a `tuple` specialization",
+                                    );
+                                    diagnostic.annotate(
+                                        self.context
+                                            .secondary(first_unpacked_variadic_tuple)
+                                            .message("First unpacked variadic tuple"),
+                                    );
+                                    diagnostic.annotate(
+                                        self.context
+                                            .secondary(element)
+                                            .message("Later unpacked variadic tuple"),
+                                    );
+                                }
+                            } else {
+                                first_unpacked_variadic_tuple = Some(element);
+                            }
+                        };
+
                         if let Some(inner_tuple) = element_ty.exact_tuple_instance_spec(self.db()) {
                             element_types = element_types.concat(self.db(), &inner_tuple);
+
+                            if inner_tuple.is_variadic() {
+                                report_too_many_unpacked_tuples();
+                            }
+                        } else if self.expression_type(starred_value)
+                            == Type::Dynamic(DynamicType::TodoTypeVarTuple)
+                        {
+                            report_too_many_unpacked_tuples();
                         } else {
                             // TODO: emit a diagnostic
                         }
@@ -623,7 +713,10 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 }
 
                 let ty = if return_todo {
-                    Some(TupleType::homogeneous(self.db(), todo_type!("PEP 646")))
+                    Some(TupleType::homogeneous(
+                        self.db(),
+                        Type::Dynamic(DynamicType::TodoTypeVarTuple),
+                    ))
                 } else {
                     TupleType::new(self.db(), &element_types.build())
                 };
@@ -631,15 +724,27 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 // Here, we store the type for the inner `int, str` tuple-expression,
                 // while the type for the outer `tuple[int, str]` slice-expression is
                 // stored in the surrounding `infer_type_expression` call:
-                self.store_expression_type(tuple_slice, Type::tuple(ty));
+                self.store_expression_type(&tuple.slice, Type::tuple(ty));
 
                 ty
             }
             single_element => {
+                if single_element.is_ellipsis_literal_expr()
+                    && let Some(builder) = self.context.report_lint(&INVALID_TYPE_FORM, tuple)
+                {
+                    let mut diagnostic = builder.into_diagnostic("Invalid `tuple` specialization");
+                    diagnostic.set_primary_message(
+                        "`...` can only be used as the second element \
+                            in a two-element `tuple` specialization",
+                    );
+                }
                 let single_element_ty = self.infer_type_expression(single_element);
                 if element_could_alter_type_of_whole_tuple(single_element, single_element_ty, self)
                 {
-                    Some(TupleType::homogeneous(self.db(), todo_type!("PEP 646")))
+                    Some(TupleType::homogeneous(
+                        self.db(),
+                        Type::Dynamic(DynamicType::TodoTypeVarTuple),
+                    ))
                 } else {
                     TupleType::heterogeneous(self.db(), std::iter::once(single_element_ty))
                 }
@@ -673,6 +778,10 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 }
                 Type::unknown()
             }
+            ast::Expr::NoneLiteral(_) => {
+                self.infer_expression(slice, TypeContext::default());
+                KnownClass::NoneType.to_subclass_of(self.db())
+            }
             ast::Expr::Subscript(
                 subscript @ ast::ExprSubscript {
                     value,
@@ -702,7 +811,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                             )
                         } else if class_literal.is_tuple(self.db()) {
                             let class_type = self
-                                .infer_tuple_type_expression(parameters)
+                                .infer_tuple_type_expression(subscript)
                                 .map(|tuple_type| tuple_type.to_class_type(self.db()))
                                 .unwrap_or_else(|| class_literal.default_specialization(self.db()));
                             SubclassOfType::from(self.db(), class_type)
@@ -710,7 +819,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                             match class_literal.generic_context(self.db()) {
                                 Some(generic_context) => {
                                     let db = self.db();
-                                    let specialize = |types: &[Option<Type<'db>>]| {
+                                    let specialize = &|types: &[Option<Type<'db>>]| {
                                         SubclassOfType::from(
                                             db,
                                             class_literal.apply_specialization(db, |_| {
@@ -793,7 +902,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                         todo_type!("specialized recursive generic type alias"),
                         generic_context.len(db),
                     )
-                    .collect(),
+                    .collect::<Vec<_>>(),
                 ),
             );
             return if in_type_expression {
@@ -805,7 +914,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
             };
         }
 
-        let specialize = |types: &[Option<Type<'db>>]| {
+        let specialize = &|types: &[Option<Type<'db>>]| {
             let specialized = value_ty.apply_specialization(
                 db,
                 generic_context.specialize_partial(db, types.iter().copied()),
@@ -921,7 +1030,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 KnownInstanceType::TypeAliasType(type_alias @ TypeAliasType::PEP695(_)) => {
                     if type_alias.specialization(self.db()).is_some() {
                         if let Some(builder) =
-                            self.context.report_lint(&NON_SUBSCRIPTABLE, subscript)
+                            self.context.report_lint(&NOT_SUBSCRIPTABLE, subscript)
                         {
                             let mut diagnostic =
                                 builder.into_diagnostic("Cannot subscript non-generic type alias");
@@ -951,7 +1060,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                             self.infer_type_expression(slice);
 
                             if let Some(builder) =
-                                self.context.report_lint(&NON_SUBSCRIPTABLE, subscript)
+                                self.context.report_lint(&NOT_SUBSCRIPTABLE, subscript)
                             {
                                 let value_type = type_alias.raw_value_type(self.db());
                                 let mut diagnostic = builder
@@ -1033,6 +1142,15 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                     }
                     Type::unknown()
                 }
+                KnownInstanceType::NamedTupleSpec(_) => {
+                    self.infer_type_expression(&subscript.slice);
+                    if let Some(builder) = self.context.report_lint(&INVALID_TYPE_FORM, subscript) {
+                        builder.into_diagnostic(format_args!(
+                            "`NamedTuple` specs cannot be specialized",
+                        ));
+                    }
+                    Type::unknown()
+                }
             },
             Type::Dynamic(DynamicType::UnknownGeneric(_)) => {
                 self.infer_explicit_type_alias_specialization(subscript, value_ty, true)
@@ -1045,12 +1163,12 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 value_ty
             }
             Type::ClassLiteral(class) => {
-                match class.generic_context(self.db()) {
-                    Some(generic_context) => {
+                match (class.generic_context(self.db()), class.as_static()) {
+                    (Some(generic_context), Some(static_class)) => {
                         let specialized_class = self.infer_explicit_class_specialization(
                             subscript,
                             value_ty,
-                            class,
+                            static_class,
                             generic_context,
                         );
 
@@ -1062,7 +1180,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                             )
                             .unwrap_or(Type::unknown())
                     }
-                    None => {
+                    _ => {
                         // TODO: emit a diagnostic if you try to specialize a non-generic class.
                         self.infer_type_expression(slice);
                         todo_type!("specialized non-generic class")
@@ -1083,13 +1201,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                     &mut self.inner_expression_inference_state,
                     InnerExpressionInferenceState::Get,
                 );
-                let union = union
-                    .elements(self.db())
-                    .iter()
-                    .fold(UnionBuilder::new(self.db()), |builder, elem| {
-                        builder.add(self.infer_subscript_type_expression(subscript, *elem))
-                    })
-                    .build();
+                let union = union.map(self.db(), |element| {
+                    self.infer_subscript_type_expression(subscript, *element)
+                });
                 self.inner_expression_inference_state = previous_slice_inference_state;
                 union
             }
@@ -1135,7 +1249,9 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
         }
         let ty = class.to_specialized_instance(
             self.db(),
-            args.iter().map(|node| self.infer_type_expression(node)),
+            args.iter()
+                .map(|node| self.infer_type_expression(node))
+                .collect::<Vec<_>>(),
         );
         if arguments.is_tuple_expr() {
             self.store_expression_type(arguments, ty);
@@ -1180,7 +1296,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
         let callable_type = if let (Some(parameters), Some(return_type), true) =
             (parameters, return_type, correct_argument_number)
         {
-            Type::single_callable(db, Signature::new(parameters, Some(return_type)))
+            Type::single_callable(db, Signature::new(parameters, return_type))
         } else {
             Type::Callable(CallableType::unknown(db))
         };
@@ -1521,10 +1637,26 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                         .top_materialization(self.db()),
                 ),
             },
-            SpecialFormType::TypeGuard => {
-                self.infer_type_expression(arguments_slice);
-                todo_type!("`TypeGuard[]` special form")
-            }
+            SpecialFormType::TypeGuard => match arguments_slice {
+                ast::Expr::Tuple(_) => {
+                    self.infer_type_expression(arguments_slice);
+
+                    if let Some(builder) = self.context.report_lint(&INVALID_TYPE_FORM, subscript) {
+                        let diag = builder.into_diagnostic(format_args!(
+                            "Special form `typing.TypeGuard` expected exactly one type parameter",
+                        ));
+                        diagnostic::add_type_expression_reference_link(diag);
+                    }
+
+                    Type::unknown()
+                }
+                _ => TypeGuardType::unbound(
+                    self.db(),
+                    // Unlike `TypeIs`, don't use top materialization, because
+                    // `TypeGuard` clobbering behavior makes it counterintuitive
+                    self.infer_type_expression(arguments_slice),
+                ),
+            },
             SpecialFormType::Concatenate => {
                 let arguments = if let ast::Expr::Tuple(tuple) = arguments_slice {
                     &*tuple.elts
@@ -1593,8 +1725,10 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
 
                     let mut argument_elements = arguments_as_tuple
                         .as_ref()
-                        .map(|tup| Either::Left(tup.all_elements().copied()))
-                        .unwrap_or(Either::Right(std::iter::once(arguments)));
+                        .map(|tup| tup.all_elements())
+                        .unwrap_or(std::slice::from_ref(&arguments))
+                        .iter()
+                        .copied();
 
                     let probably_meant_literal = argument_elements.all(|ty| match ty {
                         Type::StringLiteral(_)
@@ -1621,9 +1755,7 @@ impl<'db> TypeInferenceBuilder<'db, '_> {
                 Type::unknown()
             }
             SpecialFormType::Type => self.infer_subclass_of_type_expression(arguments_slice),
-            SpecialFormType::Tuple => {
-                Type::tuple(self.infer_tuple_type_expression(arguments_slice))
-            }
+            SpecialFormType::Tuple => Type::tuple(self.infer_tuple_type_expression(subscript)),
             SpecialFormType::Generic | SpecialFormType::Protocol => {
                 self.infer_expression(arguments_slice, TypeContext::default());
                 if let Some(builder) = self.context.report_lint(&INVALID_TYPE_FORM, subscript) {
