@@ -35,27 +35,42 @@ impl<'db> Type<'db> {
     }
 
     pub(crate) fn instance(db: &'db dyn Db, class: ClassType<'db>) -> Self {
-        let (class_literal, specialization) = class.class_literal(db);
-        match class_literal.known(db) {
-            Some(KnownClass::Tuple) => Type::tuple(TupleType::new(
-                db,
-                specialization
-                    .and_then(|spec| Some(Cow::Borrowed(spec.tuple(db)?)))
-                    .unwrap_or_else(|| Cow::Owned(TupleSpec::homogeneous(Type::unknown())))
-                    .as_ref(),
-            )),
-            Some(KnownClass::Object) => Type::object(),
-            _ => class_literal
-                .is_typed_dict(db)
-                .then(|| Type::typed_dict(class))
-                .or_else(|| {
-                    class.into_protocol_class(db).map(|protocol_class| {
-                        Self::ProtocolInstance(ProtocolInstanceType::from_class(protocol_class))
-                    })
-                })
-                .unwrap_or(Type::NominalInstance(NominalInstanceType(
-                    NominalInstanceInner::NonTuple(class),
-                ))),
+        match class.class_literal(db) {
+            // Dynamic classes created via `type()` don't have special instance types.
+            // TODO: When we add functional TypedDict support, this branch should check
+            // for TypedDict and return `Type::typed_dict(class)` for that case.
+            ClassLiteral::Dynamic(_) => {
+                Type::NominalInstance(NominalInstanceType(NominalInstanceInner::NonTuple(class)))
+            }
+            ClassLiteral::DynamicNamedTuple(_) => {
+                Type::NominalInstance(NominalInstanceType(NominalInstanceInner::NonTuple(class)))
+            }
+            ClassLiteral::Static(class_literal) => {
+                let specialization = class.into_generic_alias().map(|g| g.specialization(db));
+                match class_literal.known(db) {
+                    Some(KnownClass::Tuple) => Type::tuple(TupleType::new(
+                        db,
+                        specialization
+                            .and_then(|spec| Some(Cow::Borrowed(spec.tuple(db)?)))
+                            .unwrap_or_else(|| Cow::Owned(TupleSpec::homogeneous(Type::unknown())))
+                            .as_ref(),
+                    )),
+                    Some(KnownClass::Object) => Type::object(),
+                    _ => class_literal
+                        .is_typed_dict(db)
+                        .then(|| Type::typed_dict(class))
+                        .or_else(|| {
+                            class.into_protocol_class(db).map(|protocol_class| {
+                                Self::ProtocolInstance(ProtocolInstanceType::from_class(
+                                    protocol_class,
+                                ))
+                            })
+                        })
+                        .unwrap_or(Type::NominalInstance(NominalInstanceType(
+                            NominalInstanceInner::NonTuple(class),
+                        ))),
+                }
+            }
         }
     }
 
@@ -66,7 +81,7 @@ impl<'db> Type<'db> {
         Type::tuple_instance(tuple)
     }
 
-    pub(crate) fn homogeneous_tuple(db: &'db dyn Db, element: Type<'db>) -> Self {
+    pub fn homogeneous_tuple(db: &'db dyn Db, element: Type<'db>) -> Self {
         Type::tuple_instance(TupleType::homogeneous(db, element))
     }
 
@@ -225,18 +240,9 @@ impl<'db> NominalInstanceType<'db> {
         }
     }
 
+    /// Returns the class literal for this instance.
     pub(super) fn class_literal(&self, db: &'db dyn Db) -> ClassLiteral<'db> {
-        let class = match self.0 {
-            NominalInstanceInner::ExactTuple(tuple) => tuple.to_class_type(db),
-            NominalInstanceInner::NonTuple(class) => class,
-            NominalInstanceInner::Object => {
-                return KnownClass::Object
-                    .try_to_class_literal(db)
-                    .expect("Typeshed should always have a `object` class in `builtins.pyi`");
-            }
-        };
-        let (class_literal, _) = class.class_literal(db);
-        class_literal
+        self.class(db).class_literal(db)
     }
 
     /// Returns the [`KnownClass`] that this is a nominal instance of, or `None` if it is not an
@@ -275,7 +281,7 @@ impl<'db> NominalInstanceType<'db> {
                     .find_map(|class| match class.known(db)? {
                         // N.B. this is a pure optimisation: iterating through the MRO would give us
                         // the correct tuple spec for `sys._version_info`, since we special-case the class
-                        // in `ClassLiteral::explicit_bases()` so that it is inferred as inheriting from
+                        // in `StmtClassLiteral::explicit_bases()` so that it is inferred as inheriting from
                         // a tuple type with the correct spec for the user's configured Python version and platform.
                         KnownClass::VersionInfo => {
                             Some(Cow::Owned(TupleSpec::version_info_spec(db)))
@@ -337,10 +343,9 @@ impl<'db> NominalInstanceType<'db> {
             NominalInstanceInner::ExactTuple(_) | NominalInstanceInner::Object => return None,
             NominalInstanceInner::NonTuple(class) => class,
         };
-        let (class, Some(specialization)) = class.class_literal(db) else {
-            return None;
-        };
-        if !class.is_known(db, KnownClass::Slice) {
+        let (class_literal, specialization) = class.static_class_literal(db)?;
+        let specialization = specialization?;
+        if !class_literal.is_known(db, KnownClass::Slice) {
             return None;
         }
         let [start, stop, step] = specialization.types(db) else {
@@ -480,8 +485,13 @@ impl<'db> NominalInstanceType<'db> {
                 }
             }
         }
+
         result.or(db, || {
-            ConstraintSet::from(!(self.class(db)).could_coexist_in_mro_with(db, other.class(db)))
+            ConstraintSet::from(
+                !self
+                    .class(db)
+                    .could_coexist_in_mro_with(db, other.class(db)),
+            )
         })
     }
 
@@ -496,7 +506,7 @@ impl<'db> NominalInstanceType<'db> {
             NominalInstanceInner::NonTuple(class) => class
                 .known(db)
                 .map(KnownClass::is_singleton)
-                .unwrap_or_else(|| is_single_member_enum(db, class.class_literal(db).0)),
+                .unwrap_or_else(|| is_single_member_enum(db, class.class_literal(db))),
         }
     }
 
@@ -508,7 +518,7 @@ impl<'db> NominalInstanceType<'db> {
                 .known(db)
                 .and_then(KnownClass::is_single_valued)
                 .or_else(|| Some(self.tuple_spec(db)?.is_single_valued(db)))
-                .unwrap_or_else(|| is_single_member_enum(db, class.class_literal(db).0)),
+                .unwrap_or_else(|| is_single_member_enum(db, class.class_literal(db))),
         }
     }
 
@@ -621,7 +631,7 @@ pub(super) fn walk_protocol_instance_type<'db, V: super::visitor::TypeVisitor<'d
     } else {
         match protocol.inner {
             Protocol::FromClass(class) => {
-                if let Some(specialization) = class.class_literal(db).1 {
+                if let Some((_, Some(specialization))) = class.static_class_literal(db) {
                     walk_specialization(db, specialization, visitor);
                 }
             }
@@ -694,7 +704,7 @@ impl<'db> ProtocolInstanceType<'db> {
     /// Such a protocol is therefore an equivalent type to `object`, which would in fact be
     /// normalised to `object`.
     pub(super) fn is_equivalent_to_object(self, db: &'db dyn Db) -> bool {
-        #[salsa::tracked(cycle_initial=initial, heap_size=ruff_memory_usage::heap_size)]
+        #[salsa::tracked(cycle_initial=|_, _, _, ()| true, heap_size=ruff_memory_usage::heap_size)]
         fn is_equivalent_to_object_inner<'db>(
             db: &'db dyn Db,
             protocol: ProtocolInstanceType<'db>,
@@ -712,15 +722,6 @@ impl<'db> ProtocolInstanceType<'db> {
                 .is_always_satisfied(db)
         }
 
-        fn initial<'db>(
-            _db: &'db dyn Db,
-            _id: salsa::Id,
-            _value: ProtocolInstanceType<'db>,
-            _: (),
-        ) -> bool {
-            true
-        }
-
         is_equivalent_to_object_inner(db, self, ())
     }
 
