@@ -16,11 +16,12 @@ use ruff_db::system::SystemPathBuf;
 use serde::{Deserialize, Serialize};
 use ty_project::{Db as _, ProjectDatabase};
 
+use crate::capabilities::ResolvedClientCapabilities;
 use crate::document::{FileRangeExt, ToRangeExt};
-use crate::session::DocumentHandle;
 use crate::session::client::Client;
+use crate::session::{DocumentHandle, GlobalSettings};
 use crate::system::{AnySystemPath, file_to_url};
-use crate::{DIAGNOSTIC_NAME, Db};
+use crate::{DIAGNOSTIC_NAME, Db, DiagnosticMode};
 use crate::{PositionEncoding, Session};
 
 pub(super) struct Diagnostics {
@@ -56,7 +57,12 @@ impl Diagnostics {
         Self::result_id_from_hash(&self.items)
     }
 
-    pub(super) fn to_lsp_diagnostics(&self, db: &ProjectDatabase) -> LspDiagnostics {
+    pub(super) fn to_lsp_diagnostics(
+        &self,
+        db: &ProjectDatabase,
+        client_capabilities: ResolvedClientCapabilities,
+        global_settings: &GlobalSettings,
+    ) -> LspDiagnostics {
         if let Some(notebook_document) = db.notebook_document(self.file_or_notebook) {
             let mut cell_diagnostics: FxHashMap<Url, Vec<Diagnostic>> = FxHashMap::default();
 
@@ -67,7 +73,15 @@ impl Diagnostics {
             }
 
             for diagnostic in &self.items {
-                let (url, lsp_diagnostic) = to_lsp_diagnostic(db, diagnostic, self.encoding);
+                let Some((url, lsp_diagnostic)) = to_lsp_diagnostic(
+                    db,
+                    diagnostic,
+                    self.encoding,
+                    client_capabilities,
+                    global_settings,
+                ) else {
+                    continue;
+                };
 
                 let Some(url) = url else {
                     tracing::warn!("Unable to find notebook cell");
@@ -85,7 +99,18 @@ impl Diagnostics {
             LspDiagnostics::TextDocument(
                 self.items
                     .iter()
-                    .map(|diagnostic| to_lsp_diagnostic(db, diagnostic, self.encoding).1)
+                    .filter_map(|diagnostic| {
+                        Some(
+                            to_lsp_diagnostic(
+                                db,
+                                diagnostic,
+                                self.encoding,
+                                client_capabilities,
+                                global_settings,
+                            )?
+                            .1,
+                        )
+                    })
                     .collect(),
             )
         }
@@ -125,21 +150,7 @@ pub(super) fn clear_diagnostics_if_needed(
     {
         return;
     }
-
-    clear_diagnostics(document.url(), client);
-}
-
-/// Clears the diagnostics for the document identified by `uri`.
-///
-/// This is done by notifying the client with an empty list of diagnostics for the document.
-/// For notebook cells, this clears diagnostics for the specific cell.
-/// For other document types, this clears diagnostics for the main document.
-pub(super) fn clear_diagnostics(uri: &lsp_types::Url, client: &Client) {
-    client.send_notification::<PublishDiagnostics>(PublishDiagnosticsParams {
-        uri: uri.clone(),
-        diagnostics: vec![],
-        version: None,
-    });
+    session.clear_diagnostics(client, document.url());
 }
 
 /// Publishes the diagnostics for the given document snapshot using the [publish diagnostics
@@ -166,6 +177,10 @@ pub(super) fn publish_diagnostics_if_needed(
 /// Publishes the diagnostics for the given document snapshot using the [publish diagnostics
 /// notification].
 pub(super) fn publish_diagnostics(document: &DocumentHandle, session: &Session, client: &Client) {
+    if session.global_settings().diagnostic_mode().is_off() {
+        return;
+    }
+
     let db = session.project_db(document.notebook_or_file_path());
 
     let Some(diagnostics) = compute_diagnostics(db, document, session.position_encoding()) else {
@@ -181,7 +196,11 @@ pub(super) fn publish_diagnostics(document: &DocumentHandle, session: &Session,
         });
     };
 
-    match diagnostics.to_lsp_diagnostics(db) {
+    match diagnostics.to_lsp_diagnostics(
+        db,
+        session.client_capabilities(),
+        session.global_settings(),
+    ) {
         LspDiagnostics::TextDocument(diagnostics) => {
             publish_diagnostics_notification(document.url().clone(), diagnostics);
         }
@@ -207,53 +226,81 @@ pub(crate) fn publish_settings_diagnostics(
     // Note we DO NOT respect the fact that clients support pulls because these are
     // files they *specifically* won't pull diagnostics from us for, because we don't
     // claim to be an LSP for them.
-    if session.global_settings().diagnostic_mode().is_workspace() {
-        return;
+    match session.global_settings().diagnostic_mode() {
+        DiagnosticMode::Workspace | DiagnosticMode::Off => {
+            return;
+        }
+        DiagnosticMode::OpenFilesOnly => {}
     }
 
     let session_encoding = session.position_encoding();
-    let state = session.project_state_mut(&AnySystemPath::System(path));
-    let db = &state.db;
-    let project = db.project();
-    let settings_diagnostics = project.check_settings(db);
+    let client_capabilities = session.client_capabilities();
 
-    // We need to send diagnostics if we have non-empty ones, or we have ones to clear.
-    // These will both almost always be empty so this function will almost always be a no-op.
-    if settings_diagnostics.is_empty() && state.untracked_files_with_pushed_diagnostics.is_empty() {
-        return;
-    }
+    let project_path = AnySystemPath::System(path);
 
-    // Group diagnostics by URL
-    let mut diagnostics_by_url: FxHashMap<Url, Vec<_>> = FxHashMap::default();
-    for diagnostic in settings_diagnostics {
-        if let Some(span) = diagnostic.primary_span() {
-            let file = span.expect_ty_file();
-            let Some(url) = file_to_url(db, file) else {
-                tracing::debug!("Failed to convert file to URL at {}", file.path(db));
-                continue;
-            };
-            diagnostics_by_url.entry(url).or_default().push(diagnostic);
+    let (mut diagnostics_by_url, old_untracked) = {
+        let state = session.project_state_mut(&project_path);
+        let db = &state.db;
+        let project = db.project();
+        let settings_diagnostics = project.check_settings(db);
+
+        // We need to send diagnostics if we have non-empty ones, or we have ones to clear.
+        // These will both almost always be empty so this function will almost always be a no-op.
+        if settings_diagnostics.is_empty()
+            && state.untracked_files_with_pushed_diagnostics.is_empty()
+        {
+            return;
         }
-    }
 
-    // Record the URLs we're sending non-empty diagnostics for, so we know to clear them
-    // the next time we publish settings diagnostics!
-    let old_untracked = std::mem::replace(
-        &mut state.untracked_files_with_pushed_diagnostics,
-        diagnostics_by_url.keys().cloned().collect(),
-    );
+        // Group diagnostics by URL
+        let mut diagnostics_by_url: FxHashMap<Url, Vec<_>> = FxHashMap::default();
+        for diagnostic in settings_diagnostics {
+            if let Some(span) = diagnostic.primary_span() {
+                let file = span.expect_ty_file();
+                let Some(url) = file_to_url(db, file) else {
+                    tracing::debug!("Failed to convert file to URL at {}", file.path(db));
+                    continue;
+                };
+                diagnostics_by_url.entry(url).or_default().push(diagnostic);
+            }
+        }
+
+        // Record the URLs we're sending non-empty diagnostics for, so we know to clear them
+        // the next time we publish settings diagnostics!
+        let old_untracked = std::mem::replace(
+            &mut state.untracked_files_with_pushed_diagnostics,
+            diagnostics_by_url.keys().cloned().collect(),
+        );
+
+        (diagnostics_by_url, old_untracked)
+    };
 
     // Add empty diagnostics for any files that had diagnostics before but don't now.
     // This will clear them (either the file is no longer relevant to us or fixed!)
     for url in old_untracked {
         diagnostics_by_url.entry(url).or_default();
     }
+
+    let db = session.project_db(&project_path);
+    let global_settings = session.global_settings();
+
     // Send the settings diagnostics!
     for (url, file_diagnostics) in diagnostics_by_url {
         // Convert diagnostics to LSP format
         let lsp_diagnostics = file_diagnostics
             .into_iter()
-            .map(|diagnostic| to_lsp_diagnostic(db, &diagnostic, session_encoding).1)
+            .filter_map(|diagnostic| {
+                Some(
+                    to_lsp_diagnostic(
+                        db,
+                        &diagnostic,
+                        session_encoding,
+                        client_capabilities,
+                        global_settings,
+                    )?
+                    .1,
+                )
+            })
             .collect::<Vec<_>>();
 
         client.send_notification::<PublishDiagnostics>(PublishDiagnosticsParams {
@@ -292,7 +339,16 @@ pub(super) fn to_lsp_diagnostic(
     db: &dyn Db,
     diagnostic: &ruff_db::diagnostic::Diagnostic,
     encoding: PositionEncoding,
-) -> (Option<lsp_types::Url>, Diagnostic) {
+    client_capabilities: ResolvedClientCapabilities,
+    global_settings: &GlobalSettings,
+) -> Option<(Option<lsp_types::Url>, Diagnostic)> {
+    if diagnostic.is_invalid_syntax() && !global_settings.show_syntax_errors() {
+        return None;
+    }
+
+    let supports_related_information =
+        client_capabilities.supports_diagnostic_related_information();
+
     let location = diagnostic.primary_span().and_then(|span| {
         let file = span.expect_ty_file();
         span.range()?
@@ -330,34 +386,39 @@ pub(super) fn to_lsp_diagnostic(
         Some(CodeDescription { href })
     });
 
-    let mut related_information = Vec::new();
+    let related_information =
+        if supports_related_information {
+            let mut related_information = Vec::new();
+            related_information.extend(diagnostic.secondary_annotations().filter_map(
+                |annotation| annotation_to_related_information(db, annotation, encoding),
+            ));
 
-    related_information.extend(
-        diagnostic
-            .secondary_annotations()
-            .filter_map(|annotation| annotation_to_related_information(db, annotation, encoding)),
-    );
+            for sub_diagnostic in diagnostic.sub_diagnostics() {
+                related_information.extend(sub_diagnostic_to_related_information(
+                    db,
+                    sub_diagnostic,
+                    encoding,
+                ));
 
-    for sub_diagnostic in diagnostic.sub_diagnostics() {
-        related_information.extend(sub_diagnostic_to_related_information(
-            db,
-            sub_diagnostic,
-            encoding,
-        ));
+                related_information.extend(
+                    sub_diagnostic
+                        .annotations()
+                        .iter()
+                        .filter(|annotation| !annotation.is_primary())
+                        .filter_map(|annotation| {
+                            annotation_to_related_information(db, annotation, encoding)
+                        }),
+                );
+            }
 
-        related_information.extend(
-            sub_diagnostic
-                .annotations()
-                .iter()
-                .filter_map(|annotation| {
-                    annotation_to_related_information(db, annotation, encoding)
-                }),
-        );
-    }
+            Some(related_information)
+        } else {
+            None
+        };
 
     let data = DiagnosticData::try_from_diagnostic(db, diagnostic, encoding);
 
-    (
+    Some((
         url,
         Diagnostic {
             range,
@@ -366,11 +427,24 @@ pub(super) fn to_lsp_diagnostic(
             code: Some(NumberOrString::String(diagnostic.id().to_string())),
             code_description,
             source: Some(DIAGNOSTIC_NAME.into()),
-            message: diagnostic.concise_message().to_string(),
-            related_information: Some(related_information),
+            message: if supports_related_information {
+                // Show both the primary and annotation messages if available,
+                // because we don't create a related information for the primary message.
+                if let Some(annotation_message) = diagnostic
+                    .primary_annotation()
+                    .and_then(|annotation| annotation.get_message())
+                {
+                    format!("{}: {annotation_message}", diagnostic.primary_message())
+                } else {
+                    diagnostic.primary_message().to_string()
+                }
+            } else {
+                diagnostic.concise_message().to_string()
+            },
+            related_information,
             data: serde_json::to_value(data).ok(),
         },
-    )
+    ))
 }
 
 /// Converts an [`Annotation`] to a [`DiagnosticRelatedInformation`].
