@@ -2,6 +2,7 @@ mod cancel;
 mod did_change;
 mod did_change_notebook;
 mod did_change_watched_files;
+mod did_change_workspace_folders;
 mod did_close;
 mod did_close_notebook;
 mod did_open;
@@ -11,6 +12,7 @@ pub(super) use cancel::CancelNotificationHandler;
 pub(super) use did_change::DidChangeTextDocumentHandler;
 pub(super) use did_change_notebook::DidChangeNotebookHandler;
 pub(super) use did_change_watched_files::DidChangeWatchedFiles;
+pub(super) use did_change_workspace_folders::DidChangeWorkspaceFoldersHandler;
 pub(super) use did_close::DidCloseTextDocumentHandler;
 pub(super) use did_close_notebook::DidCloseNotebookHandler;
 pub(super) use did_open::DidOpenTextDocumentHandler;
