@@ -1,21 +1,20 @@
 use std::collections::HashMap;
 
 use lsp_types::Url;
-use ruff_db::system::SystemPathBuf;
+use ruff_db::system::{SystemPath, SystemPathBuf};
 use ruff_macros::Combine;
 use ruff_python_ast::PythonVersion;
 use serde::{Deserialize, Serialize};
-use serde_json::Value;
-
+use serde_json::{Map, Value};
 use ty_combine::Combine;
 use ty_ide::{CompletionSettings, InlayHintSettings};
 use ty_project::metadata::Options as TyOptions;
 use ty_project::metadata::options::ProjectOptionsOverrides;
-use ty_project::metadata::value::{RangedValue, RelativePathBuf};
-
-use crate::logging::LogLevel;
+use ty_project::metadata::value::{RangedValue, RelativePathBuf, ValueSource};
 
 use super::settings::{ExperimentalSettings, GlobalSettings, WorkspaceSettings};
+use crate::logging::LogLevel;
+use crate::session::client::Client;
 
 /// Initialization options that are set once at server startup that never change.
 ///
@@ -83,15 +82,15 @@ impl InitializationOptions {
 #[serde(rename_all = "camelCase")]
 pub struct ClientOptions {
     #[serde(flatten)]
-    pub(crate) global: GlobalOptions,
+    pub global: GlobalOptions,
 
     #[serde(flatten)]
-    pub(crate) workspace: WorkspaceOptions,
+    pub workspace: WorkspaceOptions,
 
     /// Additional options that aren't valid as per the schema but we accept it to provide better
     /// error message to the user.
     #[serde(flatten)]
-    pub(crate) unknown: HashMap<String, Value>,
+    pub unknown: HashMap<String, Value>,
 }
 
 impl ClientOptions {
@@ -125,6 +124,12 @@ impl ClientOptions {
         self
     }
 
+    #[must_use]
+    pub fn with_show_syntax_errors(mut self, show_syntax_errors: bool) -> Self {
+        self.global.show_syntax_errors = Some(show_syntax_errors);
+        self
+    }
+
     #[must_use]
     pub fn with_unknown(mut self, unknown: HashMap<String, Value>) -> Self {
         self.unknown = unknown;
@@ -138,12 +143,18 @@ impl ClientOptions {
 /// server.
 #[derive(Clone, Combine, Debug, Serialize, Deserialize, Default)]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct GlobalOptions {
+pub struct GlobalOptions {
     /// Diagnostic mode for the language server.
-    diagnostic_mode: Option<DiagnosticMode>,
+    pub diagnostic_mode: Option<DiagnosticMode>,
 
     /// Experimental features that the server provides on an opt-in basis.
-    pub(crate) experimental: Option<Experimental>,
+    pub experimental: Option<Experimental>,
+
+    /// If `true` or [`None`], show syntax errors as diagnostics.
+    ///
+    /// This is useful when using ty with other language servers, allowing the user to refer
+    /// to syntax errors from only one source.
+    pub show_syntax_errors: Option<bool>,
 }
 
 impl GlobalOptions {
@@ -156,6 +167,7 @@ impl GlobalOptions {
         GlobalSettings {
             diagnostic_mode: self.diagnostic_mode.unwrap_or_default(),
             experimental,
+            show_syntax_errors: self.show_syntax_errors.unwrap_or(true),
         }
     }
 }
@@ -163,31 +175,67 @@ impl GlobalOptions {
 /// Options that are specific to a workspace.
 ///
 /// These are the dynamic options that are applied to a specific workspace.
-#[derive(Clone, Combine, Debug, Serialize, Deserialize, Default)]
+#[derive(Clone, Combine, Debug, Serialize, Deserialize, Default, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct WorkspaceOptions {
+pub struct WorkspaceOptions {
+    /// Inline configuration, overrides settings from the configuration file.
+    pub configuration: Option<ConfigurationMap>,
+
+    /// Path to a `ty.toml` file, similar to `--config-file` on the CLI
+    pub configuration_file: Option<String>,
+
     /// Whether to disable language services like code completions, hover, etc.
-    disable_language_services: Option<bool>,
+    pub disable_language_services: Option<bool>,
 
     /// Options to configure inlay hints.
-    inlay_hints: Option<InlayHintOptions>,
+    pub inlay_hints: Option<InlayHintOptions>,
 
     /// Options to configure completions.
-    completions: Option<CompletionOptions>,
+    pub completions: Option<CompletionOptions>,
 
     /// Information about the currently active Python environment in the VS Code Python extension.
     ///
     /// This is relevant only for VS Code and is populated by the ty VS Code extension.
-    python_extension: Option<PythonExtension>,
+    pub python_extension: Option<PythonExtension>,
 }
 
 impl WorkspaceOptions {
-    pub(crate) fn into_settings(self) -> WorkspaceSettings {
-        let overrides = self.python_extension.and_then(|extension| {
-            let active_environment = extension.active_environment?;
+    pub(crate) fn into_settings(self, root: &SystemPath, client: &Client) -> WorkspaceSettings {
+        let configuration_file =
+            self.configuration_file
+                .and_then(|config_file| match shellexpand::full(&config_file) {
+                    Ok(path) => Some(SystemPath::absolute(&*path, root)),
+                    Err(error) => {
+                        client.show_error_message(format_args!(
+                            "Failed to expand the environment variables \
+                            for the `ty.configuration_file` setting: {error}"
+                        ));
+                        None
+                    }
+                });
 
-            let mut overrides = ProjectOptionsOverrides::new(None, TyOptions::default());
+        let options_overrides =
+            self.configuration.and_then(|map| {
+                match TyOptions::deserialize_with(
+                    ValueSource::Editor,
+                    serde::de::value::MapDeserializer::new(map.0.into_iter()),
+                ) {
+                    Ok(options) => Some(options),
+                    Err(error) => {
+                        client.show_error_message(format_args!(
+                            "Invalid `ty.configuration` options: {error}"
+                        ));
+                        None
+                    }
+                }
+            });
 
+        let mut overrides =
+            ProjectOptionsOverrides::new(configuration_file, options_overrides.unwrap_or_default());
+
+        if let Some(extension) = self.python_extension
+            && let Some(active_environment) = extension.active_environment
+        {
             overrides.fallback_python = if let Some(environment) = &active_environment.environment {
                 environment.folder_uri.to_file_path().ok().and_then(|path| {
                     Some(RelativePathBuf::python_extension(
@@ -222,9 +270,13 @@ impl WorkspaceOptions {
                     in case the configuration doesn't specify a Python version",
                 );
             }
+        }
 
+        let overrides = if overrides == ProjectOptionsOverrides::default() {
+            None
+        } else {
             Some(overrides)
-        });
+        };
 
         WorkspaceSettings {
             disable_language_services: self.disable_language_services.unwrap_or_default(),
@@ -241,9 +293,23 @@ impl WorkspaceOptions {
     }
 }
 
-#[derive(Clone, Combine, Debug, Serialize, Deserialize, Default)]
+#[derive(Clone, Debug, Serialize, Deserialize, Default, PartialEq, Eq)]
+#[serde(transparent)]
+pub struct ConfigurationMap(Map<String, Value>);
+
+impl From<Map<String, Value>> for ConfigurationMap {
+    fn from(map: Map<String, Value>) -> Self {
+        Self(map)
+    }
+}
+
+impl Combine for ConfigurationMap {
+    fn combine_with(&mut self, _other: Self) {}
+}
+
+#[derive(Clone, Combine, Debug, Serialize, Deserialize, Default, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
-struct InlayHintOptions {
+pub struct InlayHintOptions {
     variable_types: Option<bool>,
     call_argument_names: Option<bool>,
 }
@@ -257,9 +323,9 @@ impl InlayHintOptions {
     }
 }
 
-#[derive(Clone, Combine, Debug, Serialize, Deserialize, Default)]
+#[derive(Clone, Combine, Debug, Serialize, Deserialize, Default, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
-struct CompletionOptions {
+pub struct CompletionOptions {
     auto_import: Option<bool>,
 }
 
@@ -281,9 +347,13 @@ impl CompletionOptions {
 #[derive(Clone, Copy, Debug, Default, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub enum DiagnosticMode {
+    /// Disable all diagnostics so that ty can be used as an LSP only
+    Off,
+
     /// Check only currently open files.
     #[default]
     OpenFilesOnly,
+
     /// Check all files in the workspace.
     Workspace,
 }
@@ -298,6 +368,10 @@ impl DiagnosticMode {
     pub(crate) const fn is_open_files_only(self) -> bool {
         matches!(self, DiagnosticMode::OpenFilesOnly)
     }
+
+    pub(crate) const fn is_off(self) -> bool {
+        matches!(self, DiagnosticMode::Off)
+    }
 }
 
 impl Combine for DiagnosticMode {
@@ -310,15 +384,19 @@ impl Combine for DiagnosticMode {
         // So, this is a workaround to ensure that if the diagnostic mode is set to `workspace` in
         // either an initialization options or one of the workspace options, it is always set to
         // `workspace` in the global options.
-        if other.is_workspace() {
-            *self = DiagnosticMode::Workspace;
+        if other != DiagnosticMode::default() {
+            *self = other;
         }
     }
 }
 
 #[derive(Clone, Combine, Debug, Serialize, Deserialize, Default)]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct Experimental;
+#[expect(
+    clippy::empty_structs_with_brackets,
+    reason = "The LSP fails to deserialize the options when this is a unit type"
+)]
+pub struct Experimental {}
 
 impl Experimental {
     #[expect(clippy::unused_self)]
@@ -327,9 +405,9 @@ impl Experimental {
     }
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize, Default)]
+#[derive(Clone, Debug, Serialize, Deserialize, Default, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
-struct PythonExtension {
+pub struct PythonExtension {
     active_environment: Option<ActiveEnvironment>,
 }
 
@@ -342,7 +420,7 @@ impl Combine for PythonExtension {
     }
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
 pub(crate) struct ActiveEnvironment {
     pub(crate) executable: PythonExecutable,
@@ -350,7 +428,7 @@ pub(crate) struct ActiveEnvironment {
     pub(crate) version: Option<EnvironmentVersion>,
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
 pub(crate) struct EnvironmentVersion {
     pub(crate) major: i64,
@@ -361,7 +439,7 @@ pub(crate) struct EnvironmentVersion {
     pub(crate) sys_version: String,
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
 pub(crate) struct PythonEnvironment {
     pub(crate) folder_uri: Url,
@@ -372,7 +450,7 @@ pub(crate) struct PythonEnvironment {
     pub(crate) name: Option<String>,
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq)]
 #[serde(rename_all = "camelCase")]
 pub(crate) struct PythonExecutable {
     #[allow(dead_code)]
