@@ -30,6 +30,7 @@
 mod code_actions;
 mod commands;
 mod completions;
+mod configuration;
 mod initialize;
 mod inlay_hints;
 mod notebook;
@@ -38,6 +39,7 @@ mod pull_diagnostics;
 mod rename;
 mod semantic_tokens;
 mod signature_help;
+mod workspace_folders;
 
 use std::collections::{BTreeMap, HashMap, VecDeque};
 use std::num::NonZeroUsize;
@@ -51,8 +53,8 @@ use crossbeam::channel::RecvTimeoutError;
 use insta::internals::SettingsBindDropGuard;
 use lsp_server::{Connection, Message, RequestId, Response, ResponseError};
 use lsp_types::notification::{
-    DidChangeTextDocument, DidChangeWatchedFiles, DidCloseTextDocument, DidOpenTextDocument, Exit,
-    Initialized, Notification,
+    DidChangeTextDocument, DidChangeWatchedFiles, DidChangeWorkspaceFolders, DidCloseTextDocument,
+    DidOpenTextDocument, Exit, Initialized, Notification,
 };
 use lsp_types::request::{
     Completion, DocumentDiagnosticRequest, HoverRequest, Initialize, InlayHintRequest,
@@ -63,16 +65,16 @@ use lsp_types::{
     ClientCapabilities, CompletionItem, CompletionParams, CompletionResponse,
     CompletionTriggerKind, ConfigurationParams, DiagnosticClientCapabilities,
     DidChangeTextDocumentParams, DidChangeWatchedFilesClientCapabilities,
-    DidChangeWatchedFilesParams, DidCloseTextDocumentParams, DidOpenTextDocumentParams,
-    DocumentDiagnosticParams, DocumentDiagnosticReportResult, FileEvent, Hover, HoverParams,
-    InitializeParams, InitializeResult, InitializedParams, InlayHint, InlayHintClientCapabilities,
-    InlayHintParams, NumberOrString, PartialResultParams, Position, PreviousResultId,
-    PublishDiagnosticsClientCapabilities, Range, SemanticTokensResult, SignatureHelp,
-    SignatureHelpParams, SignatureHelpTriggerKind, TextDocumentClientCapabilities,
+    DidChangeWatchedFilesParams, DidChangeWorkspaceFoldersParams, DidCloseTextDocumentParams,
+    DidOpenTextDocumentParams, DocumentDiagnosticParams, DocumentDiagnosticReportResult, FileEvent,
+    Hover, HoverParams, InitializeParams, InitializeResult, InitializedParams, InlayHint,
+    InlayHintClientCapabilities, InlayHintParams, NumberOrString, PartialResultParams, Position,
+    PreviousResultId, PublishDiagnosticsClientCapabilities, Range, SemanticTokensResult,
+    SignatureHelp, SignatureHelpParams, SignatureHelpTriggerKind, TextDocumentClientCapabilities,
     TextDocumentContentChangeEvent, TextDocumentIdentifier, TextDocumentItem,
     TextDocumentPositionParams, Url, VersionedTextDocumentIdentifier, WorkDoneProgressParams,
     WorkspaceClientCapabilities, WorkspaceDiagnosticParams, WorkspaceDiagnosticReportResult,
-    WorkspaceEdit, WorkspaceFolder,
+    WorkspaceEdit, WorkspaceFolder, WorkspaceFoldersChangeEvent,
 };
 use ruff_db::system::{OsSystem, SystemPath, SystemPathBuf, TestSystem};
 use rustc_hash::FxHashMap;
@@ -211,7 +213,7 @@ impl TestServer {
         test_context: TestContext,
         capabilities: ClientCapabilities,
         initialization_options: Option<ClientOptions>,
-        env_vars: Vec<(String, String)>,
+        env_vars: Vec<(String, Option<String>)>,
     ) -> Self {
         setup_tracing();
 
@@ -225,7 +227,12 @@ impl TestServer {
         // Create test system and set environment variable overrides
         let test_system = Arc::new(TestSystem::new(os_system));
         for (name, value) in env_vars {
-            test_system.set_env_var(name, value);
+            match value {
+                Some(value) => {
+                    test_system.set_env_var(name, value);
+                }
+                None => test_system.remove_env_var(name),
+            }
         }
 
         // Start the server in a separate thread
@@ -252,7 +259,7 @@ impl TestServer {
 
         let workspace_configurations = workspaces
             .into_iter()
-            .filter_map(|(folder, options)| Some((folder.uri, options?)))
+            .map(|(folder, options)| (folder.uri, options.unwrap_or_default()))
             .collect::<HashMap<_, _>>();
 
         Self {
@@ -509,8 +516,12 @@ impl TestServer {
     /// a panic-free alternative.
     #[track_caller]
     pub(crate) fn await_notification<N: Notification>(&mut self) -> N::Params {
-        self.try_await_notification::<N>(None)
-            .unwrap_or_else(|err| panic!("Failed to receive notification `{}`: {err}", N::METHOD))
+        match self.try_await_notification::<N>(None) {
+            Ok(result) => result,
+            Err(err) => {
+                panic!("Failed to receive notification `{}`: {err}", N::METHOD)
+            }
+        }
     }
 
     /// Wait for a notification of the specified type from the server and return its parameters.
@@ -590,8 +601,12 @@ impl TestServer {
     /// If receiving the request fails.
     #[track_caller]
     pub(crate) fn await_request<R: Request>(&mut self) -> (RequestId, R::Params) {
-        self.try_await_request::<R>(None)
-            .unwrap_or_else(|err| panic!("Failed to receive server request `{}`: {err}", R::METHOD))
+        match self.try_await_request::<R>(None) {
+            Ok(result) => result,
+            Err(err) => {
+                panic!("Failed to receive server request `{}`: {err}", R::METHOD)
+            }
+        }
     }
 
     /// Wait for a request of the specified type from the server and return the request ID and
@@ -707,9 +722,9 @@ impl TestServer {
 
     /// Handle workspace configuration requests from the server.
     ///
-    /// Use the [`get_request`] method to wait for the server to send this request.
+    /// Use the [`await_request`] method to wait for the server to send this request.
     ///
-    /// [`get_request`]: TestServer::get_request
+    /// [`await_request`]: TestServer::await_request
     #[track_caller]
     fn handle_workspace_configuration_request(
         &mut self,
@@ -822,6 +837,35 @@ impl TestServer {
         self.send_notification::<DidChangeWatchedFiles>(params);
     }
 
+    /// Send a `workspace/didChangeWorkspaceFolders` notification with the given added/removed
+    /// workspace folders. The paths provided should be paths to the root of the workspace folder.
+    pub(crate) fn change_workspace_folders<P: AsRef<SystemPath>>(
+        &mut self,
+        added: impl IntoIterator<Item = P>,
+        removed: impl IntoIterator<Item = P>,
+    ) {
+        let path_to_workspace_folder = |path: &SystemPath| -> WorkspaceFolder {
+            let uri = self.file_uri(path);
+            WorkspaceFolder {
+                uri,
+                name: path.file_name().unwrap_or("").to_string(),
+            }
+        };
+        let params = DidChangeWorkspaceFoldersParams {
+            event: WorkspaceFoldersChangeEvent {
+                added: added
+                    .into_iter()
+                    .map(|path| path_to_workspace_folder(path.as_ref()))
+                    .collect(),
+                removed: removed
+                    .into_iter()
+                    .map(|path| path_to_workspace_folder(path.as_ref()))
+                    .collect(),
+            },
+        };
+        self.send_notification::<DidChangeWorkspaceFolders>(params);
+    }
+
     pub(crate) fn rename(
         &mut self,
         document: &Url,
@@ -985,6 +1029,31 @@ impl TestServer {
             },
         )
     }
+
+    /// Adds a workspace folder configuration to this wrapper's state.
+    ///
+    /// This is meant to roughly model VS Code's "Add Folder to Workspace"
+    /// functionality.
+    ///
+    /// This doesn't actually correspond to any communication with an LSP
+    /// server. To do that, you'll want `change_workspace_folders`.
+    ///
+    /// # Errors
+    ///
+    /// This can return an error when there is a problem converting the
+    /// given workspace folder root path to a URL.
+    pub(crate) fn add_workspace_folder(
+        &mut self,
+        path: &SystemPath,
+        options: Option<ClientOptions>,
+    ) -> Result<()> {
+        let path = self.test_context.root().join(path);
+        let url = Url::from_file_path(path.as_std_path())
+            .map_err(|()| anyhow!("Failed to convert workspace path to URL: {path}"))?;
+        self.workspace_configurations
+            .insert(url, options.unwrap_or_default());
+        Ok(())
+    }
 }
 
 impl fmt::Debug for TestServer {
@@ -1073,7 +1142,7 @@ pub(crate) struct TestServerBuilder {
     workspaces: Vec<(WorkspaceFolder, Option<ClientOptions>)>,
     initialization_options: Option<ClientOptions>,
     client_capabilities: ClientCapabilities,
-    env_vars: Vec<(String, String)>,
+    env_vars: Vec<(String, Option<String>)>,
 }
 
 impl TestServerBuilder {
@@ -1104,7 +1173,7 @@ impl TestServerBuilder {
             test_context: TestContext::new()?,
             initialization_options: None,
             client_capabilities,
-            env_vars: Vec::new(),
+            env_vars: vec![("VIRTUAL_ENV".to_string(), None)],
         })
     }
 
@@ -1120,7 +1189,7 @@ impl TestServerBuilder {
         name: impl Into<String>,
         value: impl Into<String>,
     ) -> Self {
-        self.env_vars.push((name.into(), value.into()));
+        self.env_vars.push((name.into(), Some(value.into())));
         self
     }
 
@@ -1136,11 +1205,6 @@ impl TestServerBuilder {
         workspace_root: &SystemPath,
         options: Option<ClientOptions>,
     ) -> Result<Self> {
-        // TODO: Support multiple workspaces in the test server
-        if self.workspaces.len() == 1 {
-            anyhow::bail!("Test server doesn't support multiple workspaces yet");
-        }
-
         let workspace_path = self.test_context.root().join(workspace_root);
         fs::create_dir_all(workspace_path.as_std_path())?;
 
@@ -1244,13 +1308,17 @@ impl TestServerBuilder {
         self
     }
 
+    pub(crate) fn file_path(&self, path: impl AsRef<SystemPath>) -> SystemPathBuf {
+        self.test_context.root().join(path)
+    }
+
     /// Write a file to the test directory
     pub(crate) fn with_file(
         self,
         path: impl AsRef<SystemPath>,
         content: impl AsRef<str>,
     ) -> Result<Self> {
-        let file_path = self.test_context.root().join(path.as_ref());
+        let file_path = self.file_path(path);
         // Ensure parent directories exists
         if let Some(parent) = file_path.parent() {
             fs::create_dir_all(parent.as_std_path())?;
@@ -1322,15 +1390,10 @@ impl TestContext {
         })?;
 
         let mut settings = insta::Settings::clone_current();
+        let project_dir_url = Url::from_file_path(project_dir.as_std_path())
+            .map_err(|()| anyhow!("Failed to convert root directory to url"))?;
         settings.add_filter(&tempdir_filter(project_dir.as_str()), "<temp_dir>/");
-        settings.add_filter(
-            &tempdir_filter(
-                Url::from_file_path(project_dir.as_std_path())
-                    .map_err(|()| anyhow!("Failed to convert root directory to url"))?
-                    .path(),
-            ),
-            "<temp_dir>/",
-        );
+        settings.add_filter(&tempdir_filter(project_dir_url.path()), "<temp_dir>/");
         settings.add_filter(r#"\\(\w\w|\s|\.|")"#, "/$1");
         settings.add_filter(
             r#"The system cannot find the file specified."#,
