@@ -208,8 +208,11 @@ fn discard_todo_metadata(ty: &str) -> Cow<'_, str> {
     {
         /// `@Todo` variants that are hardcoded and always display their message,
         /// even in release mode.
-        const PRESERVED_TODO_VARIANTS: &[&str] =
-            &["@Todo(StarredExpression)", "@Todo(typing.Unpack)"];
+        const PRESERVED_TODO_VARIANTS: &[&str] = &[
+            "@Todo(StarredExpression)",
+            "@Todo(typing.Unpack)",
+            "@Todo(TypeVarTuple)",
+        ];
 
         static TODO_METADATA_REGEX: LazyLock<regex::Regex> =
             LazyLock::new(|| regex::Regex::new(r"@Todo\([^)]*\)").unwrap());
